import time

import requests
from dateutil.relativedelta import relativedelta

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        uprn = kwargs.get("uprn")
        usrn = kwargs.get("paon")
        check_uprn(uprn)
        check_usrn(usrn)
        bindata = {"bins": []}

        SESSION_URL = "https://crawleybc-self.achieveservice.com/authapi/isauthenticated?uri=https%253A%252F%252Fcrawleybc-self.achieveservice.com%252Fen%252FAchieveForms%252F%253Fform_uri%253Dsandbox-publish%253A%252F%252FAF-Process-fb73f73e-e8f5-4441-9f83-8b5d04d889d6%252FAF-Stage-ec9ada91-d2d9-43bc-9730-597d15fc8108%252Fdefinition.json%2526redirectlink%253D%252Fen%2526cancelRedirectLink%253D%252Fen%2526noLoginPrompt%253D1%2526accept%253Dyes&hostname=crawleybc-self.achieveservice.com&withCredentials=true"

        API_URL = "https://crawleybc-self.achieveservice.com/apibroker/"

        currentdate = datetime.now().strftime("%d/%m/%Y")

        data = {
            "formValues": {
                "Address": {
                    "address": {
                        "value": {
                            "Address": {
                                "usrn": {
                                    "value": usrn,
                                },
                                "uprn": {
                                    "value": uprn,
                                },
                            }
                        },
                    },
                    "dayConverted": {
                        "value": currentdate,
                    },
                    "getCollection": {
                        "value": "true",
                    },
                    "getWorksheets": {
                        "value": "false",
                    },
                },
            },
        }

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "User-Agent": "Mozilla/5.0",
            "X-Requested-With": "XMLHttpRequest",
            "Referer": "https://crawleybc-self.achieveservice.com/fillform/?iframe_id=fillform-frame-1&db_id=",
        }
        s = requests.session()
        r = s.get(SESSION_URL)
        r.raise_for_status()
        session_data = r.json()
        sid = session_data["auth-session"]
        params = {
            "api": "RunLookup",
            "id": "5b4f0ec5f13f4",
            "repeat_against": "",
            "noRetry": "true",
            "getOnlyTokens": "undefined",
            "log_id": "",
            "app_name": "AF-Renderer::Self",
            # unix_timestamp
            "_": str(int(time.time() * 1000)),
            "sid": sid,
        }

        r = s.post(API_URL, json=data, headers=headers, params=params)
        r.raise_for_status()

        data = r.json()
        rows_data = data["integration"]["transformed"]["rows_data"]["0"]
        if not isinstance(rows_data, dict):
            raise ValueError("Invalid data returned from API")

        # Extract each service's relevant details for the bin schedule
        for key, value in rows_data.items():
            if key.endswith("DateNext"):
                BinType = key.replace("DateNext", "Service")
                for key2, value2 in rows_data.items():
                    if key2 == BinType:
                        BinType = value2
                next_collection = datetime.strptime(value, "%A %d %B").replace(
                    year=datetime.now().year
                )
                if datetime.now().month == 12 and next_collection.month == 1:
                    next_collection = next_collection + relativedelta(years=1)

                dict_data = {
                    "type": BinType,
                    "collectionDate": next_collection.strftime(date_format),
                }
                bindata["bins"].append(dict_data)

        return bindata



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        bindata = {"bins": []}

        street = user_uprn.split(",")[0]
        suburb = user_uprn.split(",")[1]

        if not street:
            print("Street name could not be parsed"),
            return
        if not suburb:
            print("Street name could not be parsed"),
            return

        wastes = {
            "Non recycleable (green bin)",
            "Garden waste (brown bin)",
            "Paper and packaging (grey bin)",
        }
        DATE_REGEX = "^([0-9]{1,2} [A-Za-z]+ [0-9]{4})"

        s = requests.Session()
        # Loop through waste types
        for waste in wastes:
            r = s.get(
                f"https://ww3.wirral.gov.uk//recycling/detailContentDru7.asp?s={street}&t={suburb}&c={waste}"
            )
            # extract dates
            soup = BeautifulSoup(r.text, "html.parser")
            dates = soup.findAll("li")
            if len(dates) != 0:
                for item in dates:
                    match = re.match(DATE_REGEX, item.text)
                    if match:
                        dict_data = {
                            "type": waste,
                            "collectionDate": datetime.strptime(
                                match.group(1),
                                "%d %B %Y",
                            ).strftime("%d/%m/%Y"),
                        }
                        bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



from datetime import datetime

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    def parse_data(self, page: str, **kwargs) -> dict:
        uprn = kwargs.get("uprn")
        check_uprn(uprn)

        # Figure bin data URL from UPRN
        url = "https://www.northyorks.gov.uk/bin-calendar/lookup"
        payload = {
            "selected_address": uprn,
            "submit": "Continue",
            "form_id": "bin_calendar_lookup_form",
        }
        headers = {"Content-Type": "application/x-www-form-urlencoded"}

        # This endpoint redirects to the data url.
        response = requests.request("POST", url, headers=headers, data=payload)
        bin_data_url = f"{response.url}/ajax"

        # Get bin data
        response = requests.request("GET", bin_data_url)
        bin_data = response.json()

        # Parse bin data
        soup = BeautifulSoup(bin_data[1]["data"], "html.parser")

        # All collection info is in the table
        table = (
            soup.find("div", {"id": "upcoming-collection"}).find("table").find("tbody")
        )
        rows = table.find_all("tr")

        data = {"bins": []}

        for row in rows:
            cols = row.find_all("td")
            # First column is date
            bin_date = datetime.strptime(cols[0].text.strip(), "%d %B %Y")

            # Third column may contain multiple bin types separated by line breaks
            # .stripped_strings yields a generator over all non-whitespace text segments
            bin_types = [txt for txt in cols[2].stripped_strings]

            for sub_bin in bin_types:
                data["bins"].append(
                    {
                        "type": sub_bin,
                        "collectionDate": bin_date.strftime(date_format),
                    }
                )

        return data



import logging
import pickle
import time

import requests
from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from uk_bin_collection.uk_bin_collection.common import *

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# Set up logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)


class CouncilClass(AbstractGetBinDataClass):

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            collections = []
            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_postcode(user_postcode)
            url = "https://forms.newforest.gov.uk/ufs/FIND_MY_COLLECTION.eb"

            # Get session cookies using requests

            user_agent = """general.useragent.override", "userAgent=Mozilla/5.0 
            (iPhone; CPU iPhone OS 15_4 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like 
            Gecko) CriOS/101.0.4951.44 Mobile/15E148 Safari/604.1"""

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, user_agent, __name__)

            # Navigate to the page first
            driver.get(url)
            driver.refresh()  # important otherwise it results in too many redirects

            wait = WebDriverWait(driver, 60)

            logging.info("Entering postcode")
            input_element_postcode = wait.until(
                EC.presence_of_element_located((By.XPATH, '//input[@id="CTID-1-_-A"]'))
            )

            input_element_postcode.send_keys(user_postcode)

            logging.info("Searching for postcode")
            input_element_postcode_btn = wait.until(
                EC.element_to_be_clickable((By.XPATH, '//input[@type="submit"]'))
            )

            input_element_postcode_btn.click()

            logging.info("Waiting for address dropdown")
            input_element_postcode_dropdown = wait.until(
                EC.presence_of_element_located((By.XPATH, '//select[@id="CTID-6-_-A"]'))
            )

            logging.info("Selecting address")
            drop_down_values = Select(input_element_postcode_dropdown)
            option_element = wait.until(
                EC.presence_of_element_located(
                    (By.CSS_SELECTOR, f'option[value="{str(user_uprn)}"]')
                )
            )

            driver.execute_script("arguments[0].scrollIntoView();", option_element)
            drop_down_values.select_by_value(str(user_uprn))

            input_element_address_btn = wait.until(
                EC.element_to_be_clickable((By.XPATH, '//input[@value="Submit"]'))
            )

            input_element_address_btn.click()

            logging.info("Waiting for bin collection page")
            h4_element = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, "//h1[contains(text(), 'Collections days for')]")
                )
            )

            logging.info("Extracting bin collection data")
            soup = BeautifulSoup(driver.page_source, features="html.parser")
            bins = []

            # Rubbish and recycling
            rubbish_recycling = soup.find(
                "span", class_="CTID-77-_ eb-77-Override-textControl"
            )
            if rubbish_recycling:
                match = re.search(r"collected weekly on (\w+)", rubbish_recycling.text)
                if match:
                    day_name = match.group(1)
                    next_collection = get_next_day_of_week(day_name)
                    bins.append(
                        {
                            "type": "Rubbish and recycling",
                            "collectionDate": next_collection,
                        }
                    )

            # Glass collection
            glass_collection = soup.find("span", class_="CTID-78-_ eb-78-textControl")
            if glass_collection:
                match = re.search(
                    r"next collection is\s+(\d{2}/\d{2}/\d{4})", glass_collection.text
                )
                if match:
                    bins.append(
                        {"type": "Glass collection", "collectionDate": match.group(1)}
                    )

            # Garden waste
            garden_waste = soup.find("div", class_="eb-2HIpCnWC-Override-EditorInput")
            if garden_waste:
                match = re.search(
                    r"(\d{2}/\d{2}/\d{4})", garden_waste.text
                )
                if match:
                    bins.append(
                        {"type": "Garden waste", "collectionDate": match.group(1)}
                    )

            return {"bins": bins}

        except Exception as e:
            logging.error(f"An error occurred: {e}")
            raise

        finally:
            if driver:
                driver.quit()



import urllib.parse

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        api_url = f"https://api.westnorthants.digital/openapi/v1/unified-waste-collections/{user_uprn}"
        json_data = requests.get(api_url).json()

        data = {"bins": []}

        collections = json_data["collectionItems"]
        for collection in collections:
            dict_data = {
                "type": collection["type"].capitalize(),
                "collectionDate": datetime.strptime(
                    collection["date"], "%Y-%m-%d"
                ).strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



import logging
import time

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# Set up logging
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)


class CouncilClass(AbstractGetBinDataClass):
    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            collections = []
            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_postcode(user_postcode)

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            if not headless:
                driver.set_window_size(1920, 1080)

            driver.get(
                "https://my.threerivers.gov.uk/en/AchieveForms/?mode=fill&consentMessage=yes&form_uri=sandbox-publish://AF-Process-52df96e3-992a-4b39-bba3-06cfaabcb42b/AF-Stage-01ee28aa-1584-442c-8d1f-119b6e27114a/definition.json&process=1&process_uri=sandbox-processes://AF-Process-52df96e3-992a-4b39-bba3-06cfaabcb42b&process_id=AF-Process-52df96e3-992a-4b39-bba3-06cfaabcb42b&noLoginPrompt=1"
            )
            wait = WebDriverWait(driver, 60)

            def click_element(by, value):
                element = wait.until(EC.element_to_be_clickable((by, value)))
                driver.execute_script("arguments[0].scrollIntoView();", element)
                element.click()

            click_element(By.XPATH, "//button[contains(text(), 'Continue')]")

            logging.info("Switching to iframe")
            iframe_presence = wait.until(
                EC.presence_of_element_located((By.ID, "fillform-frame-1"))
            )
            driver.switch_to.frame(iframe_presence)

            logging.info("Entering postcode")
            input_element_postcode = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, '//input[@id="postcode_search"]')
                )
            )
            input_element_postcode.send_keys(user_postcode)

            logging.info("Selecting address")
            dropdown = wait.until(EC.element_to_be_clickable((By.ID, "chooseAddress")))
            dropdown_options = wait.until(
                EC.presence_of_element_located((By.CLASS_NAME, "lookup-option"))
            )
            drop_down_values = Select(dropdown)
            option_element = wait.until(
                EC.presence_of_element_located(
                    (By.CSS_SELECTOR, f'option.lookup-option[value="{str(user_uprn)}"]')
                )
            )

            driver.execute_script("arguments[0].scrollIntoView();", option_element)
            drop_down_values.select_by_value(str(user_uprn))

            option_element = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, '//div[@class="fieldContent"][1]')
                )
            )

            time.sleep(2)

            click_element(By.XPATH, "//button/span[contains(text(), 'Next')]")

            logging.info("Waiting for bin schedule")
            bin_results = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, "//div[@data-field-name='subCollectionCalendar']//table")
                )
            )

            logging.info("Extracting bin collection data")
            soup = BeautifulSoup(driver.page_source, features="html.parser")
            bin_cards = soup.find_all(
                "div", {"data-field-name": "subCollectionCalendar"}
            )

            bins = []

            for bin_card in bin_cards:
                # Try to find the table within the bin_card
                table = bin_card.find(
                    "table",
                    {
                        "class": "repeatable-table table table-responsive table-hover table-condensed"
                    },
                )

                if table:
                    print("Table found")
                    rows = table.select("tr.repeatable-value")
                    for row in rows:
                        cols = row.find_all("td", class_="value")
                        if len(cols) >= 3:  # Ensure there are enough columns
                            bin_type = cols[1].find_all("span")[-1].text.strip()
                            collection_date = (
                                cols[2]
                                .find_all("span")[-1]
                                .text.strip()
                                .replace("-", "/")
                            )
                            bins.append(
                                {"type": bin_type, "collectionDate": collection_date}
                            )
                else:
                    print("Table not found within bin_card")

            bin_data = {"bins": bins}
            logging.info("Data extraction complete")
            return bin_data

        except Exception as e:
            logging.error(f"An error occurred: {e}")
            raise

        finally:
            if driver:
                driver.quit()



import requests
from bs4 import BeautifulSoup
from dateutil.relativedelta import relativedelta

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        bindata = {"bins": []}
        curr_date = datetime.today()

        soup = BeautifulSoup(page.content, features="html.parser")
        button = soup.find("a", text="Find out which bin will be collected when and sign up for a free email reminder.")

        if button["href"]:
            URI = button["href"]
            # Make the GET request
            response = requests.get(URI)
            soup = BeautifulSoup(response.content, features="html.parser")
            divs = soup.find_all("div", {"class": "editor"})
            for div in divs:
                lis = div.find_all("li")
                for li in lis:
                    collection = li.text.split(": ")
                    collection_date = datetime.strptime(
                        collection[0],
                        "%A %d %B",
                    ).replace(year=curr_date.year)
                    if curr_date.month == 12 and collection_date.month == 1:
                        collection_date = collection_date + relativedelta(years=1)
                    bin_types = collection[1].split(" and ")
                    for bin_type in bin_types:
                        dict_data = {
                            "type": bin_type,
                            "collectionDate": collection_date.strftime("%d/%m/%Y"),
                        }
                        bindata["bins"].append(dict_data)
        else:
            print("Failed to find bin schedule")

        return bindata



import re
import time

import requests
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        collection_day = kwargs.get("paon")
        garden_collection_week = kwargs.get("postcode")
        garden_collection_day = kwargs.get("uprn")
        bindata = {"bins": []}

        days_of_week = [
            "Monday",
            "Tuesday",
            "Wednesday",
            "Thursday",
            "Friday",
            "Saturday",
            "Sunday",
        ]

        garden_week = ["Week 1", "Week 2"]

        refusestartDate = datetime(2024, 11, 11)
        recyclingstartDate = datetime(2024, 11, 4)

        offset_days = days_of_week.index(collection_day)
        offset_days_garden = days_of_week.index(garden_collection_day)
        if garden_collection_week:
            garden_collection = garden_week.index(garden_collection_week)

        refuse_dates = get_dates_every_x_days(refusestartDate, 14, 28)
        recycling_dates = get_dates_every_x_days(recyclingstartDate, 14, 28)

        bank_holidays = [
            ("25/12/2024", 2),
            ("26/12/2024", 2),
            ("27/12/2024", 3),
            ("30/12/2024", 1),
            ("31/12/2024", 2),
            ("01/01/2025", 2),
            ("02/01/2025", 2),
            ("03/01/2025", 3),
            ("06/01/2025", 1),
            ("07/01/2025", 1),
            ("08/01/2025", 1),
            ("09/01/2025", 1),
            ("10/01/2025", 1),
            ("18/04/2025", 1),
            ("21/04/2025", 1),
            ("22/04/2025", 1),
            ("23/04/2025", 1),
            ("24/04/2025", 1),
            ("25/04/2025", 1),
            ("05/05/2025", 1),
            ("06/05/2025", 1),
            ("07/05/2025", 1),
            ("08/05/2025", 1),
            ("09/05/2025", 1),
            ("26/05/2025", 1),
            ("27/05/2025", 1),
            ("28/05/2025", 1),
            ("29/05/2025", 1),
            ("30/05/2025", 1),
            ("25/08/2025", 1),
            ("26/08/2025", 1),
            ("27/08/2025", 1),
            ("28/08/2025", 1),
            ("29/08/2025", 1),
        ]

        for refuseDate in refuse_dates:

            collection_date = (
                datetime.strptime(refuseDate, "%d/%m/%Y") + timedelta(days=offset_days)
            ).strftime("%d/%m/%Y")

            holiday_offset = next(
                (value for date, value in bank_holidays if date == collection_date), 0
            )

            if holiday_offset > 0:
                collection_date = (
                    datetime.strptime(collection_date, "%d/%m/%Y")
                    + timedelta(days=holiday_offset)
                ).strftime("%d/%m/%Y")

            dict_data = {
                "type": "Refuse Bin",
                "collectionDate": collection_date,
            }
            bindata["bins"].append(dict_data)

        for recyclingDate in recycling_dates:

            collection_date = (
                datetime.strptime(recyclingDate, "%d/%m/%Y")
                + timedelta(days=offset_days)
            ).strftime("%d/%m/%Y")

            holiday_offset = next(
                (value for date, value in bank_holidays if date == collection_date), 0
            )

            if holiday_offset > 0:
                collection_date = (
                    datetime.strptime(collection_date, "%d/%m/%Y")
                    + timedelta(days=holiday_offset)
                ).strftime("%d/%m/%Y")

            dict_data = {
                "type": "Recycling Bin",
                "collectionDate": collection_date,
            }
            bindata["bins"].append(dict_data)

        if garden_collection_week:
            if garden_collection == 0:
                gardenstartDate = datetime(2024, 11, 11)
            elif garden_collection == 1:
                gardenstartDate = datetime(2024, 11, 4)

            garden_dates = get_dates_every_x_days(gardenstartDate, 14, 28)

            garden_bank_holidays = [
                ("23/12/2024", 1),
                ("24/12/2024", 1),
                ("25/12/2024", 1),
                ("26/12/2024", 1),
                ("27/12/2024", 1),
                ("30/12/2024", 1),
                ("31/12/2024", 1),
                ("01/01/2025", 1),
                ("02/01/2025", 1),
                ("03/01/2025", 1),
            ]

            for gardenDate in garden_dates:

                collection_date = (
                    datetime.strptime(gardenDate, "%d/%m/%Y")
                    + timedelta(days=offset_days_garden)
                ).strftime("%d/%m/%Y")

                garden_holiday = next(
                    (
                        value
                        for date, value in garden_bank_holidays
                        if date == collection_date
                    ),
                    0,
                )

                if garden_holiday > 0:
                    continue

                holiday_offset = next(
                    (value for date, value in bank_holidays if date == collection_date),
                    0,
                )

                if holiday_offset > 0:
                    collection_date = (
                        datetime.strptime(collection_date, "%d/%m/%Y")
                        + timedelta(days=holiday_offset)
                    ).strftime("%d/%m/%Y")

                dict_data = {
                    "type": "Garden Bin",
                    "collectionDate": collection_date,
                }
                bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



from datetime import datetime, timedelta

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://www.runnymede.gov.uk/homepage/150/check-your-bin-collection-day?address={user_uprn}"

        # Make the GET request
        response = requests.get(URI)

        soup = BeautifulSoup(response.text, "html.parser")

        div = soup.find("div", class_="widget-bin-collection")

        table = div.find("table")

        tbody = table.find("tbody")

        for tr in tbody.find_all("tr"):
            tds = tr.find_all("td")
            bin_type = tds[0].text.strip()
            date_text = tds[1].text.strip()

            dict_data = {
                "type": bin_type,
                "collectionDate": (
                    datetime.strptime(date_text, "%A, %d %B %Y")
                ).strftime(date_format),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return bindata



import logging

from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        checkValid = soup.find("p", id="selectedAddressResult")
        if checkValid is None:
            raise ValueError("Address/UPRN not found")

        collections = soup.find("div", id="wasteCollectionDates")

        for bins in collections.select('div[class*="hc-island"]'):
            bin_type = bins.h4.get_text(strip=True)

            # Last div.hc-island is the calendar link, skip it
            if bin_type == "Calendar":
                continue

            # Next collection date is in a span under the second p.hc-no-margin of the div.
            bin_collection = re.search(
                r"(.*) \(.*\)", bins.select("div > p > span")[0].get_text(strip=True)
            ).group(1)
            if bin_collection:
                logging.info(
                    f"Bin type: {bin_type} - Collection date: {bin_collection}"
                )
                dict_data = {
                    "type": bin_type,
                    "collectionDate": datetime.strptime(
                        bin_collection, "%A %d %B %Y"
                    ).strftime(date_format),
                }
                data["bins"].append(dict_data)

        return data



import time
from datetime import datetime

from bs4 import BeautifulSoup
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select, WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

import re

class CouncilClass(AbstractGetBinDataClass):
    def parse_data(self, page: str, **kwargs) -> dict:
        try:
            data = {"bins": []}

            user_paon = kwargs.get("paon")
            headless = kwargs.get("headless")
            web_driver = kwargs.get("web_driver")
            driver = create_webdriver(web_driver, headless, None, __name__)

            page = "https://www.middlesbrough.gov.uk/recycling-and-rubbish/bin-collection-dates/"
            driver.get(page)

            address_box = WebDriverWait(driver, timeout=15).until(
                EC.presence_of_element_located((By.ID, "row-input-0"))
            )
            address_box.click()
            address_box.send_keys(user_paon)

            search_button = WebDriverWait(driver, timeout=15).until(
                EC.presence_of_element_located((By.ID, "rCbtn-search"))
            )
            search_button.click()

            iframe_presense = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located((By.ID, "recollect-frame"))
            )
            driver.switch_to.frame(iframe_presense)

            results = WebDriverWait(driver, timeout=15).until(
                EC.presence_of_element_located((By.ID, "rCpage-place_calendar"))
            )

            html_content = driver.page_source
            soup = BeautifulSoup(html_content, "html.parser")

            calendar_section = soup.find("section", {"id": "alt-calendar-list"})
            if not calendar_section:
                raise ValueError("Calendar section not found in the HTML.")

            date_headers = calendar_section.find_all("h3")
            collection_lists = calendar_section.find_all("ul")

            current_month = datetime.now().month
            current_year = datetime.now().year

            for date_header, collection_list in zip(date_headers, collection_lists):
                raw_date = date_header.text.strip()

                # **Regex to match "Wednesday, February 19" format**
                match = re.match(r"([A-Za-z]+), ([A-Za-z]+) (\d{1,2})", raw_date)
                
                if match:
                    day_name, month_name, day_number = match.groups()  # Extract components
                    extracted_month = datetime.strptime(month_name, "%B").month
                    extracted_day = int(day_number)

                    # Handle Dec-Jan rollover: If month is before the current month, assume next year
                    inferred_year = current_year + 1 if extracted_month < current_month else current_year

                    # **Correct the raw_date format before parsing**
                    raw_date = f"{day_name}, {month_name} {day_number}, {inferred_year}"

                print(f"DEBUG: Final raw_date before parsing -> {raw_date}")  # Debugging output

                # Convert to required format (%d/%m/%Y)
                try:
                    parsed_date = datetime.strptime(raw_date, "%A, %B %d, %Y")
                    formatted_date = parsed_date.strftime(date_format)
                except ValueError:
                    raise ValueError(f"Date format error after inference: {raw_date}")

                for li in collection_list.find_all("li"):
                    bin_type = li.get_text(strip=True).split(".")[0]
                    data["bins"].append(
                        {"type": bin_type, "collectionDate": formatted_date}
                    )

        except Exception as e:
            print(f"An error occurred: {e}")
            raise
        finally:
            if driver:
                driver.quit()

        return data



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            uri = "https://selfserve.derbyshiredales.gov.uk/renderform.aspx?t=103&k=9644C066D2168A4C21BCDA351DA2642526359DFF"

            bindata = {"bins": []}

            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            check_uprn(user_uprn)
            check_postcode(user_postcode)

            # Start a session
            session = requests.Session()

            response = session.get(uri)

            soup = BeautifulSoup(response.content, features="html.parser")

            # Function to extract hidden input values
            def get_hidden_value(soup, name):
                element = soup.find("input", {"name": name})
                return element["value"] if element else None

            # Extract the required values
            data = {
                "__RequestVerificationToken": get_hidden_value(
                    soup, "__RequestVerificationToken"
                ),
                "FormGuid": get_hidden_value(soup, "FormGuid"),
                "ObjectTemplateID": get_hidden_value(soup, "ObjectTemplateID"),
                "Trigger": "submit",
                "CurrentSectionID": get_hidden_value(soup, "CurrentSectionID"),
                "TriggerCtl": "",
                "FF2924": "U" + user_uprn,
                "FF2924lbltxt": "Collection address",
                "FF2924-text": user_postcode,
            }

            # Print extracted data
            # print("Extracted Data:", data)

            # Step 2: Submit the extracted data via a POST request
            headers = {
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/120.0.0.0 Safari/537.36",
                "Referer": uri,
                "Content-Type": "application/x-www-form-urlencoded",
            }

            URI = "https://selfserve.derbyshiredales.gov.uk/renderform/Form"

            # Make the POST request
            post_response = session.post(URI, data=data, headers=headers)

            soup = BeautifulSoup(post_response.content, features="html.parser")

            # print(soup)

            bin_rows = soup.find("div", {"class": "ss_confPanel"})

            bin_rows = bin_rows.find_all("div", {"class": "row"})
            if bin_rows:
                for bin_row in bin_rows:
                    bin_data = bin_row.find_all("div")
                    if bin_data and bin_data[0] and bin_data[1]:
                        if bin_data[0].get_text(strip=True) == "Your Collections":
                            continue
                        collection_date = datetime.strptime(
                            bin_data[0].get_text(strip=True), "%A%d %B, %Y"
                        )
                        dict_data = {
                            "type": bin_data[1].get_text(strip=True),
                            "collectionDate": collection_date.strftime(date_format),
                        }
                        bindata["bins"].append(dict_data)

            bindata["bins"].sort(
                key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
            )
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return bindata



import time
from datetime import datetime

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://portal.walthamforest.gov.uk/AchieveForms/?mode=fill&consentMessage=yes&form_uri=sandbox-publish://AF-Process-d62ccdd2-3de9-48eb-a229-8e20cbdd6393/AF-Stage-8bf39bf9-5391-4c24-857f-0dc2025c67f4/definition.json&process=1&process_uri=sandbox-processes://AF-Process-d62ccdd2-3de9-48eb-a229-8e20cbdd6393&process_id=AF-Process-d62ccdd2-3de9-48eb-a229-8e20cbdd6393"

            user_postcode = kwargs.get("postcode")
            user_uprn = kwargs.get("uprn")
            user_paon = kwargs.get("paon")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")

            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            iframe_presense = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located((By.ID, "fillform-frame-1"))
            )

            driver.switch_to.frame(iframe_presense)
            wait = WebDriverWait(driver, 60)
            inputElement_postcodesearch = wait.until(
                EC.element_to_be_clickable((By.NAME, "postcode_search"))
            )

            inputElement_postcodesearch.send_keys(user_postcode)
            find_address_button = wait.until(
                EC.element_to_be_clickable((By.ID, "lookupPostcode"))
            )

            find_address_button.send_keys(Keys.RETURN)

            dropdown = wait.until(
                EC.element_to_be_clickable((By.CLASS_NAME, "select2-choice"))
            )

            time.sleep(1)
            dropdown.click()

            dropdown_search = wait.until(
                EC.element_to_be_clickable((By.CLASS_NAME, "select2-input"))
            )
            dropdown_search.click()

            dropdown_search.send_keys(user_paon)
            dropdown_search.send_keys(Keys.RETURN)

            find_ac_button = wait.until(
                EC.element_to_be_clickable((By.ID, "confirmSearchUPRN"))
            )

            find_ac_button.send_keys(Keys.RETURN)
            h4_element = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, "//h4[contains(text(), 'Next Collections')]")
                )
            )

            data_table = WebDriverWait(driver, 10).until(
                EC.presence_of_element_located(
                    (
                        By.XPATH,
                        '//div[contains(@class, "fieldContent")]',
                    )
                )
            )
            # Make a BS4 object
            soup = BeautifulSoup(driver.page_source, features="html.parser")

            data = {"bins": []}

            collection_divs = soup.find_all("div", {"style": "text-align: center;"})

            for collection_div in collection_divs:
                h5_tag = collection_div.find("h5")
                p_tag = collection_div.find("p")

                if h5_tag and p_tag:
                    bin_type = h5_tag.get_text(strip=True)
                    collection_date_text = p_tag.find("b").get_text(strip=True)

                    # Extract and format the date
                    date_match = re.search(r"(\d+ \w+)", collection_date_text)
                    if date_match:
                        date_str = date_match.group(1)
                        date_obj = datetime.strptime(
                            date_str + " " + str(datetime.today().year), "%d %B %Y"
                        )
                        collection_date = get_next_occurrence_from_day_month(
                            date_obj
                        ).strftime(date_format)

                        data["bins"].append(
                            {"type": bin_type, "collectionDate": collection_date}
                        )

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



import time

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """
    
    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        SESSION_URL = "https://integration.aberdeencity.gov.uk/authapi/isauthenticated?uri=https%253A%252F%252Fintegration.aberdeencity.gov.uk%252Fservice%252Fbin_collection_calendar___view&hostname=integration.aberdeencity.gov.uk&withCredentials=true"

        API_URL = "https://integration.aberdeencity.gov.uk/apibroker/runLookup"

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "User-Agent": "Mozilla/5.0",
            "X-Requested-With": "XMLHttpRequest",
            "Referer": "https://integration.aberdeencity.gov.uk/fillform/?iframe_id=fillform-frame-1&db_id=",
        }
        s = requests.session()
        r = s.get(SESSION_URL)
        r.raise_for_status()
        session_data = r.json()
        sid = session_data["auth-session"]
        params = {
            "id": "583c08ffc47fe",
            "repeat_against": "",
            "noRetry": "true",
            "getOnlyTokens": "undefined",
            "log_id": "",
            "app_name": "AF-Renderer::Self",
            # unix_timestamp
            "_": str(int(time.time() * 1000)),
            "sid": sid,
        }

        r = s.post(API_URL, headers=headers, params=params)
        r.raise_for_status()

        data = r.json()
        rows_data = data["integration"]["transformed"]["rows_data"]["0"]
        if not isinstance(rows_data, dict):
            raise ValueError("Invalid data returned from API")
        token = rows_data["token"]

        data = {
            "formValues": {
                "Section 1": {
                    "nauprn": {
                        "value": user_uprn,
                    },
                    "token": {
                        "value": token,
                    },
                    "mindate": {
                        "value": datetime.now().strftime("%Y-%m-%d"),
                    },
                    "maxdate": {
                        "value": (datetime.now() + timedelta(days=30)).strftime(
                            "%Y-%m-%d"
                        ),
                    },
                },
            },
        }

        params = {
            "id": "5a3141caf4016",
            "repeat_against": "",
            "noRetry": "true",
            "getOnlyTokens": "undefined",
            "log_id": "",
            "app_name": "AF-Renderer::Self",
            # unix_timestamp
            "_": str(int(time.time() * 1000)),
            "sid": sid,
        }

        r = s.post(API_URL, json=data, headers=headers, params=params)
        r.raise_for_status()

        data = r.json()
        rows_data = data["integration"]["transformed"]["rows_data"]["0"]
        if not isinstance(rows_data, dict):
            raise ValueError("Invalid data returned from API")

        date_pattern = re.compile(r"^(.*?)(Date\d+)$")
        count_pattern = re.compile(r"^Count(.*)$")
        for key, value in rows_data.items():
            date_match = date_pattern.match(key)
            # Match count keys
            count_match = count_pattern.match(key)
            if count_match:
                continue

            # Match date keys
            date_match = date_pattern.match(key)
            if date_match:
                bin_type = date_match.group(1)
                dict_data = {
                    "type": bin_type,
                    "collectionDate": datetime.strptime(value, "%A %d %B %Y").strftime(
                        date_format
                    ),
                }
                bindata["bins"].append(dict_data)

        return bindata



import json

import requests
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)

        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
        }
        params = {
            "type": "JSON",
            "list": "DomesticBinCollections",
            "Road": "",
            "Postcode": user_postcode,
        }

        response = requests.get(
            "https://www.fareham.gov.uk/internetlookups/search_data.aspx",
            params=params,
            headers=headers,
        )

        bin_data = response.json()["data"]
        data = {"bins": []}

        if "rows" in bin_data:
            collection_str = bin_data["rows"][0]["DomesticBinDay"]

            results = re.findall(r"(\d\d?\/\d\d?\/\d{4}) \((\w*)\)", collection_str)

            if results:
                for result in results:
                    collection_date = datetime.strptime(result[0], "%d/%m/%Y")
                    dict_data = {
                        "type": result[1],
                        "collectionDate": collection_date.strftime(date_format),
                    }
                    data["bins"].append(dict_data)

                    # Garden waste is also collected on recycling day
                    if dict_data["type"] == "Recycling":
                        garden_data = {
                            "type": "Garden",
                            "collectionDate": dict_data["collectionDate"],
                        }
                        data["bins"].append(garden_data)
            else:
                raise RuntimeError("Dates not parsed correctly.")
        else:
            raise ValueError("Postcode not found on website.")

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return data



import time

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://www.thanet.gov.uk/wp-content/mu-plugins/collection-day/incl/mu-collection-day-calls.php?pAddress={user_uprn}"

        headers = {
            "x-requested-with": "XMLHttpRequest",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36",
        }

        # Make the GET request
        response = requests.get(URI, headers=headers)

        # Parse the JSON response
        bin_collection = response.json()

        # Loop through each collection in bin_collection
        for collection in bin_collection:
            bin_type = collection["type"]
            collection_date = collection["nextDate"].split(" ")[0]

            dict_data = {
                "type": bin_type,
                "collectionDate": collection_date,
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = (
            f"https://maps.monmouthshire.gov.uk/?action=SetAddress&UniqueId={user_uprn}"
        )

        # Make the GET request
        response = requests.get(URI)

        # Parse the HTML
        soup = BeautifulSoup(response.content, "html.parser")

        waste_collections_div = soup.find("div", {"aria-label": "Waste Collections"})

        # Find all bin collection panels
        bin_panels = waste_collections_div.find_all("div", class_="atPanelContent")

        current_year = datetime.now().year
        current_month = datetime.now().month

        for panel in bin_panels:
            # Extract bin name (e.g., "Household rubbish bag")
            bin_name = panel.find("h4").text.strip().replace("\r", "").replace("\n", "")

            # Extract collection date (e.g., "Monday 9th December")
            date_tag = panel.find("p")
            if date_tag and "Your next collection date is" in date_tag.text.strip().replace("\r", "").replace("\n", ""):
                collection_date = date_tag.find("strong").text.strip()
            else:
                continue

            collection_date = datetime.strptime(
                remove_ordinal_indicator_from_date_string(collection_date), "%A %d %B"
            )

            if (current_month > 9) and (collection_date.month < 4):
                collection_date = collection_date.replace(year=(current_year + 1))
            else:
                collection_date = collection_date.replace(year=current_year)

            dict_data = {
                "type": bin_name,
                "collectionDate": collection_date.strftime("%d/%m/%Y"),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import time
from datetime import datetime

from selenium.webdriver.support.ui import Select
from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            # Make a BS4 object

            page = "https://www.chichester.gov.uk/checkyourbinday"

            user_postcode = kwargs.get("postcode")
            user_uprn = kwargs.get("uprn")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            house_number = kwargs.get("paon")

            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            wait = WebDriverWait(driver, 60)

            inputElement_postcodesearch = wait.until(
                EC.visibility_of_element_located(
                    (By.ID, "WASTECOLLECTIONCALENDARV5_CALENDAR_ADDRESSLOOKUPPOSTCODE")
                )
            )

            inputElement_postcodesearch.send_keys(user_postcode)

            inputElement_postcodesearch_btn = wait.until(
                EC.visibility_of_element_located(
                    (By.ID, "WASTECOLLECTIONCALENDARV5_CALENDAR_ADDRESSLOOKUPSEARCH")
                )
            )
            inputElement_postcodesearch_btn.send_keys(Keys.ENTER)

            inputElement_select_address = wait.until(
                EC.element_to_be_clickable(
                    (By.ID, "WASTECOLLECTIONCALENDARV5_CALENDAR_ADDRESSLOOKUPADDRESS")
                )
            )
            dropdown_element = driver.find_element(
                By.ID, "WASTECOLLECTIONCALENDARV5_CALENDAR_ADDRESSLOOKUPADDRESS"
            )

            # Now create a Select object based on the found element
            dropdown = Select(dropdown_element)

            # Select the option by visible text
            dropdown.select_by_visible_text(house_number)

            results = wait.until(
                EC.element_to_be_clickable(
                    (By.CLASS_NAME, "bin-collection-dates-container")
                )
            )

            soup = BeautifulSoup(driver.page_source, features="html.parser")
            soup.prettify()

            # Extract data from the table
            bin_collection_data = []
            rows = soup.find(
                "table", class_="defaultgeneral bin-collection-dates"
            ).find_all("tr")
            for row in rows:
                cells = row.find_all("td")
                if cells:
                    date_str = cells[0].text.strip()
                    bin_type = cells[1].text.strip()
                    # Convert date string to the required format DD/MM/YYYY
                    date_obj = datetime.strptime(date_str, "%d %B %Y")
                    date_formatted = date_obj.strftime(date_format)
                    bin_collection_data.append(
                        {"collectionDate": date_formatted, "type": bin_type}
                    )

            # Convert to JSON
            json_data = {"bins": bin_collection_data}

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return json_data



# import re

import requests

from uk_bin_collection.uk_bin_collection.common import (
    check_postcode,
    check_uprn,
    datetime,
    get_dates_every_x_days,
    json,
    timedelta,
)
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs: str) -> dict[str, list[dict[str, str]]]:
        if (postcode := kwargs.get("postcode")) is None:
            raise KeyError("Missing: postcode")
        if (uprn := kwargs.get("uprn")) is None:
            raise KeyError("Missing: uprn")
        check_postcode(postcode)
        check_uprn(uprn)
        bindata: dict[str, list[dict[str, str]]] = {"bins": []}
        location_x: int = 0
        location_y: int = 0
        location_usrn: str = ""

        # Ensure any cookies set are maintained in a requests session
        s = requests.session()

        # Ask for a new SessionId from the server
        session_id_url = "https://maps.cheltenham.gov.uk/map/Aurora.svc/"\
            "RequestSession?userName=guest+CBC&password=&"\
            "script=%5CAurora%5CCBC+Waste+Streets.AuroraScript%24"
        session_id_response = s.get(session_id_url)
        session_id_response.raise_for_status()
        session_id = session_id_response.json().get("Session").get("SessionId")

        # Ask what tasks we can do within the session
        tasks_url = f"https://maps.cheltenham.gov.uk/map/Aurora.svc/"\
            f"GetWorkflow?sessionId={session_id}&workflowId=wastestreet"
        tasks_response = s.get(tasks_url)
        tasks_response.raise_for_status()
        # JSON response contained a BOM marker
        tasks = json.loads(tasks_response.text[1:])
        retrieve_results_task_id, initialise_map_task_id, drilldown_task_id = None, None, None
        # Pull out the ID's of the tasks we will need
        for task in tasks.get("Tasks"):
            if task.get("$type") == "StatMap.Aurora.FetchResultSetTask, StatMapService":
                retrieve_results_task_id = task.get("Id")
            elif task.get("$type") == "StatMap.Aurora.SaveStateTask, StatMapService":
                initialise_map_task_id = task.get("Id")
            elif task.get("$type") == "StatMap.Aurora.DrillDownTask, StatMapService":
                drilldown_task_id = task.get("Id")
        if not all([retrieve_results_task_id, initialise_map_task_id, drilldown_task_id]):
            raise ValueError("Not all task ID's found")

        # Find the X / Y coordinates for the requested postcode
        postcode_search_url = "https://maps.cheltenham.gov.uk/map/Aurora.svc/FindLocation?"\
            f"sessionId={session_id}&address={postcode}&limit=1000"
        postcode_search_response = s.get(postcode_search_url)
        postcode_search_response.raise_for_status()
        if len(locations_list := postcode_search_response.json().get("Locations")) == 0:
            raise ValueError("Address locations empty")
        for location in locations_list:
            location_search_url = "https://maps.cheltenham.gov.uk/map/Aurora.svc/FindLocation?"\
                f"sessionId={session_id}&locationId={location.get('Id')}"
            location_search_response = s.get(location_search_url)
            location_search_response.raise_for_status()
            if not (location_list := location_search_response.json().get("Locations")):
                raise KeyError("Locations wasn't present in results")
            if not (location_detail := location_list[0].get("Details")):
                raise KeyError("Details wasn't present in location")
            location_uprn = [detail.get(
                "Value") for detail in location_detail if detail.get("Name") == "UPRN"][0]
            if str(location_uprn) == uprn:
                location_usrn = str([detail.get(
                    "Value") for detail in location_detail if detail.get("Name") == "USRN"][0])
                location_x = location_list[0].get("X")
                location_y = location_list[0].get("Y")
                break

        # Needed to initialise the server to allow follow on call
        open_map_url = "https://maps.cheltenham.gov.uk/map/Aurora.svc/OpenScriptMap?"\
            f"sessionId={session_id}"
        if res := s.get(open_map_url):
            res.raise_for_status()

        # Needed to initialise the server to allow follow on call
        save_state_map_url = "https://maps.cheltenham.gov.uk/map/Aurora.svc/ExecuteTaskJob?"\
            f"sessionId={session_id}&taskId={initialise_map_task_id}&job="\
            "%7BTask%3A+%7B+%24type%3A+%27StatMap.Aurora.SaveStateTask%2C"\
            "+StatMapService%27+%7D%7D"
        if res := s.get(save_state_map_url):
            res.raise_for_status()

        # Start search for address given by x / y coord
        drilldown_map_url = "https://maps.cheltenham.gov.uk/map/Aurora.svc/ExecuteTaskJob?"\
            f"sessionId={session_id}&taskId={drilldown_task_id}&job=%7B%22"\
            f"QueryX%22%3A{location_x}%2C%22QueryY%22%3A{location_y}%2C%22"\
            "Task%22%3A%7B%22Type%22%3A%22StatMap.Aurora.DrillDownTask%2C"\
            "+StatMapService%22%7D%7D"
        if res := s.get(drilldown_map_url):
            res.raise_for_status()

        # Get results from search for address given by x / y coord
        address_details_url = "https://maps.cheltenham.gov.uk/map/Aurora.svc/ExecuteTaskJob?"\
            f"sessionId={session_id}&taskId={retrieve_results_task_id}"\
            f"&job=%7B%22QueryX%22%3A{location_x}%2C%22QueryY%22%3A"\
            f"{location_y}%2C%22Task%22%3A%7B%22Type%22%3A%22"\
            "StatMap.Aurora.FetchResultSetTask%2C+StatMapService"\
            "%22%2C%22ResultSetName%22%3A%22inspection%22%7D%7D"
        address_details_response = s.get(address_details_url)
        address_details_response.raise_for_status()
        # JSON response contained a BOM marker, skip first character
        address_details = json.loads(address_details_response.text[1:])
        if not (task_results := address_details.get("TaskResult")):
            raise KeyError("TaskResult wasn't present in results")
        if not (distance_export_set := task_results.get("DistanceOrderedSet")):
            raise KeyError("DistanceOrderedSet wasn't present in TaskResult")
        if not (result_set := distance_export_set.get("ResultSet")):
            raise KeyError("ResultSet wasn't present in DistanceOrderedSet")
        if not (result_tables := result_set.get("Tables")):
            raise KeyError("Tables wasn't present in ResultSet")
        result = result_tables[0]
        column_names: dict[int, str] = {}
        result_dict: dict[str, str | int] = {}
        for column in result.get("ColumnDefinitions"):
            column_names[column.get("ColumnIndex")] = column.get("ColumnName")
        for r in result.get("Records"):
            result_dict: dict[str, str | int] = {}
            for idx, column_value in enumerate(r):
                if not (column_name := column_names.get(idx)):
                    raise IndexError("Column index out of range")
                result_dict[column_name.upper()] = column_value
            # Validate the street against the USRN. Some locations can return multiple results.
            # Break on first match of USRN
            # TODO: Need to select the correct option out of all available options
            if location_usrn == str(result_dict.get("USRN")):
                break

        refuse_week, recycling_week, garden_week = 0, 0, 0
        # After we've got the correct result, pull out the week number each bin type is taken on
        if (refuse_week_raw := result_dict.get("New_Refuse_Week".upper())) is not None:
            refuse_week = int(refuse_week_raw)
        if (recycling_week_raw := result_dict.get("New_Recycling_Week".upper())) is not None:
            recycling_week = int(recycling_week_raw)
        if (garden_week_raw := result_dict.get("Garden_Bin_Week".upper())) is not None:
            garden_week = int(garden_week_raw)

        if not all([refuse_week, recycling_week, garden_week]):
            raise KeyError("Not all week numbers found")

        days_of_week = [
            "MON",
            "TUE",
            "WED",
            "THU",
            "FRI",
            "SAT",
            "SUN",
        ]

        refuse_day_offset = days_of_week.index(
            str(result_dict.get("New_Refuse_Day_internal".upper())).upper())
        recycling_day_offset = days_of_week.index(
            str(result_dict.get("New_Recycling_Day".upper())).upper())
        garden_day_offset = days_of_week.index(
            str(result_dict.get("New_Garden_Day".upper())).upper())
        food_day_offset = days_of_week.index(
            str(result_dict.get("New_Food_Day".upper())).upper())

        # Initialise WEEK-1/WEEK-2 based on known details
        week_1_epoch = datetime(2025, 1, 13)

        # Start of this week
        this_week = datetime.now() - timedelta(days=datetime.now().weekday())

        # If there's an even number of weeks between the week-1
        # epoch and this week, then this week is of type week-1
        if (((this_week - week_1_epoch).days // 7)) % 2 == 0:
            week = {
                1: this_week,
                2: this_week + timedelta(days=7)
            }
        else:
            week = {
                1: this_week - timedelta(days=7),
                2: this_week
            }

        refuse_dates: list[str] = get_dates_every_x_days(
            week[refuse_week], 14, 28)
        recycling_dates: list[str] = get_dates_every_x_days(
            week[recycling_week], 14, 28)
        garden_dates: list[str] = get_dates_every_x_days(
            week[garden_week], 14, 28)

        for refuse_date in refuse_dates:
            collection_date = (
                datetime.strptime(refuse_date, "%d/%m/%Y") +
                timedelta(days=refuse_day_offset)
            ).strftime("%d/%m/%Y")

            dict_data = {
                "type": "Refuse Bin",
                "collectionDate": collection_date,
            }
            bindata["bins"].append(dict_data)

        for recycling_date in recycling_dates:

            collection_date = (
                datetime.strptime(recycling_date, "%d/%m/%Y") +
                timedelta(days=recycling_day_offset)
            ).strftime("%d/%m/%Y")

            dict_data = {
                "type": "Recycling Bin",
                "collectionDate": collection_date,
            }
            bindata["bins"].append(dict_data)

        for garden_date in garden_dates:

            collection_date = (
                datetime.strptime(garden_date, "%d/%m/%Y") +
                timedelta(days=garden_day_offset)
            ).strftime("%d/%m/%Y")

            dict_data = {
                "type": "Garden Waste Bin",
                "collectionDate": collection_date,
            }
            bindata["bins"].append(dict_data)

        if ((food_waste_week := str(result_dict.get("FOOD_WASTE_WEEK_EXTERNAL", "")).upper())
                == "weekly".upper()):
            food_dates: list[str] = get_dates_every_x_days(week[1], 7, 56)

            for food_date in food_dates:

                collection_date = (
                    datetime.strptime(food_date, "%d/%m/%Y") +
                    timedelta(days=food_day_offset)
                ).strftime("%d/%m/%Y")

                dict_data = {
                    "type": "Food Waste Bin",
                    "collectionDate": collection_date,
                }
                bindata["bins"].append(dict_data)
        # Sometimes the food bin is collected on different days between
        # week-1 and week-2
        else:
            first_week: str | int
            second_week_detail: str
            first_week, _, second_week_detail = food_waste_week.partition("&")
            first_week = int(first_week.strip())

            second_week_day, _, second_week_number = second_week_detail.partition(
                "WEEK")
            second_week_number = int(second_week_number.strip())
            second_week_day: str = second_week_day.strip()[:3]

            food_dates_first: list[str] = get_dates_every_x_days(
                week[first_week], 14, 28)
            food_dates_second: list[str] = get_dates_every_x_days(
                week[second_week_number], 14, 28)
            second_week_offset = days_of_week.index(second_week_day)

            for food_date in food_dates_first:

                collection_date = (
                    datetime.strptime(food_date, "%d/%m/%Y") +
                    timedelta(days=food_day_offset)
                ).strftime("%d/%m/%Y")

                dict_data = {
                    "type": "Food Waste Bin",
                    "collectionDate": collection_date,
                }
                bindata["bins"].append(dict_data)
            for food_date in food_dates_second:

                collection_date = (
                    datetime.strptime(food_date, "%d/%m/%Y") +
                    timedelta(days=second_week_offset)
                ).strftime("%d/%m/%Y")

                dict_data = {
                    "type": "Food Waste Bin",
                    "collectionDate": collection_date,
                }
                bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(
                x.get("collectionDate", ""), "%d/%m/%Y")
        )
        return bindata



import pytest

# Test the command-line options


def test_headless_mode(pytestconfig):
    # Simulate pytest command-line option
    headless_mode_value = pytestconfig.getoption("--headless")
    assert headless_mode_value == "True"  # This should match the default value


def test_local_browser(pytestconfig):
    local_browser_value = pytestconfig.getoption("--local_browser")
    assert local_browser_value == "False"  # This should match the default value


def test_selenium_url(pytestconfig):
    selenium_url_value = pytestconfig.getoption("--selenium_url")
    assert (
        selenium_url_value == "http://localhost:4444"
    )  # This should match the default value


# Test the fixtures


def test_headless_mode_fixture(headless_mode):
    assert headless_mode == "True"  # This should match the default value


def test_local_browser_fixture(local_browser):
    assert local_browser == "False"  # This should match the default value


def test_selenium_url_fixture(selenium_url):
    assert (
        selenium_url == "http://localhost:4444"
    )  # This should match the default value



from typing import Dict, Any, Optional
from bs4 import BeautifulSoup, Tag, NavigableString
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

"""
This module provides bin collection data for Cheshire East Council.
"""

class CouncilClass(AbstractGetBinDataClass):
    """
    A class to fetch and parse bin collection data for Cheshire East Council.
    """

    def parse_data(self, page: Any, **kwargs: Any) -> Dict[str, Any]:
        soup = BeautifulSoup(page.text, features="html.parser")

        bin_data_dict: Dict[str, Any] = {"bins": []}

        table: Optional[Tag | NavigableString] = soup.find(
            "table", {"class": "job-details"}
        )

        if isinstance(table, Tag):  # Ensure we only proceed if 'table' is a Tag
            rows = table.find_all("tr", {"class": "data-row"})

            for row in rows:
                cells = row.find_all(
                    "td",
                    {
                        "class": lambda L: isinstance(L, str)
                        and L.startswith("visible-cell")
                    },  # Explicitly check if L is a string
                )
                labels: list[Tag] = cells[0].find_all("label") if cells else []

                if len(labels) >= 3:
                    bin_type: str = labels[2].get_text(strip=True)
                    collection_date: str = labels[1].get_text(strip=True)

                    bin_data_dict["bins"].append(
                        {
                            "type": bin_type,
                            "collectionDate": collection_date,
                        }
                    )

        return bin_data_dict



import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        data = {"bins": []}

        baseurl = "https://maps.southderbyshire.gov.uk/iShareLIVE.web//getdata.aspx?RequestType=LocalInfo&ms=mapsources/MyHouse&format=JSONP&group=Recycling%20Bins%20and%20Waste|Next%20Bin%20Collections&uid="
        url = baseurl + user_uprn

        # Make the web request
        response = requests.get(url).text

        # Remove the JSONP wrapper using a regular expression
        jsonp_pattern = r"\{.*\}"
        json_match = re.search(jsonp_pattern, response)

        if json_match:
            # Extract the JSON part
            json_data = json_match.group(0)

            # Parse the JSON
            parsed_data = json.loads(json_data)

            # Extract the embedded HTML string
            html_content = parsed_data["Results"]["Next_Bin_Collections"]["_"]

            # Parse the HTML to extract dates and bin types using regex
            matches = re.findall(
                r"<span.*?>(\d{2} \w+ \d{4})</span>.*?<span.*?>(.*?)</span>",
                html_content,
                re.S,
            )

            # Output the parsed bin collection details
            for match in matches:
                dict_data = {
                    "type": match[1],
                    "collectionDate": datetime.strptime(match[0], "%d %B %Y").strftime(
                        "%d/%m/%Y"
                    ),
                }
                data["bins"].append(dict_data)
        else:
            print("No valid JSON found in the response.")

        return data



from time import sleep

import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        # check_uprn(user_uprn)
        bindata = {"bins": []}

        URI = f"https://waste-services.sutton.gov.uk/waste/{user_uprn}"

        s = requests.Session()

        r = s.get(URI)
        while "Loading your bin days..." in r.text:
            sleep(2)
            r = s.get(URI)
        r.raise_for_status()

        soup = BeautifulSoup(r.content, "html.parser")

        current_year = datetime.now().year
        next_year = current_year + 1

        services = soup.find_all("h3", class_="govuk-heading-m waste-service-name")

        for service in services:
            bin_type = service.get_text(
                strip=True
            )  # Bin type name (e.g., 'Food waste', 'Mixed recycling')
            if bin_type == "Bulky Waste":
                continue
            service_details = service.find_next("div", class_="govuk-grid-row")

            next_collection = (
                (
                    service_details.find("dt", string="Next collection")
                    .find_next_sibling("dd")
                    .get_text(strip=True)
                )
                .replace("(this collection has been adjusted from its usual time)", "")
                .strip()
            )

            next_collection = datetime.strptime(
                remove_ordinal_indicator_from_date_string(next_collection),
                "%A, %d %B",
            )

            if (datetime.now().month == 12) and (next_collection.month == 1):
                next_collection = next_collection.replace(year=next_year)
            else:
                next_collection = next_collection.replace(year=current_year)

            dict_data = {
                "type": bin_type,
                "collectionDate": next_collection.strftime("%d/%m/%Y"),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), "%d/%m/%Y")
        )

        return bindata



import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_postcode = kwargs.get("postcode")
        user_paon = kwargs.get("paon")
        check_postcode(user_postcode)
        check_paon(user_paon)
        bindata = {"bins": []}

        URI = "https://uhtn-wrp.whitespacews.com/"

        session = requests.Session()

        # get link from first page as has some kind of unique hash
        r = session.get(
            URI,
        )
        r.raise_for_status()
        soup = BeautifulSoup(r.text, features="html.parser")

        alink = soup.find("a", text="Find my bin collection day")

        if alink is None:
            raise Exception("Initial page did not load correctly")

        # greplace 'seq' query string to skip next step
        nextpageurl = alink["href"].replace("seq=1", "seq=2")

        data = {
            "address_name_number": user_paon,
            "address_postcode": user_postcode,
        }

        # get list of addresses
        r = session.post(nextpageurl, data)
        r.raise_for_status()

        soup = BeautifulSoup(r.text, features="html.parser")

        # get first address (if you don't enter enough argument values this won't find the right address)
        alink = soup.find("div", id="property_list").find("a")

        if alink is None:
            raise Exception("Address not found")

        nextpageurl = URI + alink["href"]

        # get collection page
        r = session.get(
            nextpageurl,
        )
        r.raise_for_status()
        soup = BeautifulSoup(r.text, features="html.parser")

        if soup.find("span", id="waste-hint"):
            raise Exception("No scheduled services at this address")

        u1s = soup.find("section", id="scheduled-collections").find_all("u1")

        for u1 in u1s:
            lis = u1.find_all("li", recursive=False)

            date = lis[1].text.replace("\n", "")
            bin_type = lis[2].text.replace("\n", "")

            dict_data = {
                "type": bin_type,
                "collectionDate": datetime.strptime(
                    date,
                    "%d/%m/%Y",
                ).strftime(date_format),
            }
            bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return bindata



from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
from dateutil import parser


class CouncilClass(AbstractGetBinDataClass):
    def parse_data(self, page: str, **kwargs) -> dict:
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        data = {"bins": []}

        headers = {
            "Content-Type": "application/json; charset=UTF-8",
        }

        req_data = {"uprn": user_uprn, "noticeBoard": "default"}

        url = "https://gis.stalbans.gov.uk/NoticeBoard9/VeoliaProxy.NoticeBoard.asmx/GetServicesByUprnAndNoticeBoard"

        response = requests.post(url, json=req_data, headers=headers)

        collections_response = response.json()

        collections = []

        for collection in collections_response["d"]:
            collection_data = collection["ServiceHeaders"][0]
            bin_type = collection_data["TaskType"]
            collection_date = collection_data["Next"]
            next_collection = parser.isoparser().isoparse(collection_date)
            collections.append((bin_type, next_collection))

        ordered_data = sorted(collections, key=lambda x: x[1])

        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



import time

import requests

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        SESSION_URL = "https://highland-self.achieveservice.com/authapi/isauthenticated?uri=https%3A%2F%2Fhighland-self.achieveservice.com%2Fen%2Fservice%2FCheck_your_household_bin_collection_days&hostname=highland-self.achieveservice.com&withCredentials=true"

        API_URL = "https://highland-self.achieveservice.com/apibroker/runLookup"

        data = {
            "formValues": {"Your address": {"propertyuprn": {"value": user_uprn}}},
        }
        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "User-Agent": "Mozilla/5.0",
            "X-Requested-With": "XMLHttpRequest",
            "Referer": "https://highland-self.achieveservice.com/fillform/?iframe_id=fillform-frame-1&db_id=",
        }
        s = requests.session()
        r = s.get(SESSION_URL)
        r.raise_for_status()
        session_data = r.json()
        sid = session_data["auth-session"]
        params = {
            "id": "660d44a698632",
            "repeat_against": "",
            "noRetry": "false",
            "getOnlyTokens": "undefined",
            "log_id": "",
            "app_name": "AF-Renderer::Self",
            # unix_timestamp
            "_": str(int(time.time() * 1000)),
            "sid": sid,
        }

        r = s.post(API_URL, json=data, headers=headers, params=params)
        r.raise_for_status()

        data = r.json()
        rows_data = data["integration"]["transformed"]["rows_data"]["0"]
        if not isinstance(rows_data, dict):
            raise ValueError("Invalid data returned from API")

        use_new = any(k.endswith("New") and v for k, v in rows_data.items())
        next_date_key = "NextDateNew" if use_new else "NextDateOld"

        for key, value in rows_data.items():
            if not (key.endswith("NextDate") or key.endswith(next_date_key)):
                continue

            bin_type = key.split("NextDate")[0]
            if bin_type == "refuse":
                bin_type = "Non-recyclable waste"
            if bin_type == "fibres":
                bin_type = "Paper, card and cardboard recycling"
            if bin_type == "containers":
                bin_type = "Plastics, metals and cartons recycling"
            if bin_type == "garden":
                bin_type = "Garden waste"
            if bin_type == "food":
                bin_type = "Food waste"

            try:
                date = datetime.strptime(value, "%Y-%m-%d").strftime("%d/%m/%Y")
            except ValueError:
                continue
            dict_data = {"type": bin_type, "collectionDate": date}
            bindata["bins"].append(dict_data)

        return bindata



import time
from datetime import datetime

from bs4 import BeautifulSoup
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# import the wonderful Beautiful Soup and the URL grabber


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            page = "https://gloucester-self.achieveservice.com/service/Bins___Check_your_bin_day"

            bin_data = {"bins": []}

            user_uprn = kwargs.get("uprn")
            user_postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            check_uprn(user_uprn)
            check_postcode(user_postcode)
            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless, None, __name__)
            driver.get(page)

            cookies_button = WebDriverWait(driver, timeout=15).until(
                EC.presence_of_element_located((By.ID, "close-cookie-message"))
            )
            cookies_button.click()

            without_login_button = WebDriverWait(driver, timeout=15).until(
                EC.presence_of_element_located(
                    (By.LINK_TEXT, "or, Continue with no account")
                )
            )
            without_login_button.click()

            iframe_presense = WebDriverWait(driver, 30).until(
                EC.presence_of_element_located((By.ID, "fillform-frame-1"))
            )

            driver.switch_to.frame(iframe_presense)
            wait = WebDriverWait(driver, 60)
            inputElement_postcodesearch = wait.until(
                EC.element_to_be_clickable((By.NAME, "find_postcode"))
            )

            inputElement_postcodesearch.send_keys(user_postcode)

            # Wait for the 'Select address' dropdown to be updated
            time.sleep(2)

            dropdown = wait.until(
                EC.element_to_be_clickable((By.NAME, "chooseAddress"))
            )
            # Create a 'Select' for it, then select the first address in the list
            # (Index 0 is "Select...")
            dropdownSelect = Select(dropdown)
            dropdownSelect.select_by_value(str(user_uprn))

            # Wait for 'Searching for...' to be added to page
            WebDriverWait(driver, timeout=15).until(
                EC.text_to_be_present_in_element(
                    (By.CSS_SELECTOR, "span[data-name=html1]"), "Searching"
                )
            )

            # Wait for 'Searching for...' to be removed from page
            WebDriverWait(driver, timeout=15).until(
                EC.none_of(
                    EC.text_to_be_present_in_element(
                        (By.CSS_SELECTOR, "span[data-name=html1]"), "Searching"
                    )
                )
            )

            # Even then it can still be adding data to the page...
            time.sleep(5)

            soup = BeautifulSoup(driver.page_source, features="html.parser")

            # This is ugly but there is literally no consistency to the HTML
            def is_a_collection_date(t):
                return any("Next collection" in c for c in t.children)

            for next_collection in soup.find_all(is_a_collection_date):
                bin_info = list(
                    next_collection.parent.select_one("div:nth-child(1)").children
                )
                if not bin_info:
                    continue
                bin = bin_info[0].get_text()
                date = next_collection.select_one("strong").get_text(strip=True)
                bin_date = datetime.strptime(date, "%d %b %Y")
                dict_data = {
                    "type": bin,
                    "collectionDate": bin_date.strftime(date_format),
                }
                bin_data["bins"].append(dict_data)

            bin_data["bins"].sort(
                key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
            )

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return bin_data



import time

import requests
from dateutil.relativedelta import relativedelta

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:

        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)
        bindata = {"bins": []}

        SESSION_URL = "https://my.gravesham.gov.uk/authapi/isauthenticated?uri=https%253A%252F%252Fmy.gravesham.gov.uk%252Fen%252FAchieveForms%252F%253Fform_uri%253Dsandbox-publish%253A%252F%252FAF-Process-22218d5c-c6d6-492f-b627-c713771126be%252FAF-Stage-905e87c1-144b-4a72-8932-5518ddd3e618%252Fdefinition.json%2526redirectlink%253D%25252Fen%2526cancelRedirectLink%253D%25252Fen%2526consentMessage%253Dyes&hostname=my.gravesham.gov.uk&withCredentials=true"

        API_URL = "https://my.gravesham.gov.uk/apibroker/runLookup"

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
            "User-Agent": "Mozilla/5.0",
            "X-Requested-With": "XMLHttpRequest",
            "Referer": "https://my.gravesham.gov.uk/fillform/?iframe_id=fillform-frame-1&db_id=",
        }
        s = requests.session()
        r = s.get(SESSION_URL)
        r.raise_for_status()
        session_data = r.json()
        sid = session_data["auth-session"]
        params = {
            "id": "5ee8854759297",
            "repeat_against": "",
            "noRetry": "false",
            "getOnlyTokens": "undefined",
            "log_id": "",
            "app_name": "AF-Renderer::Self",
            # unix_timestamp
            "_": str(int(time.time() * 1000)),
            "sid": sid,
        }
        r = s.post(API_URL, headers=headers, params=params)
        r.raise_for_status()

        data = r.json()
        rows_data = data["integration"]["transformed"]["rows_data"]["0"]
        tokenString = rows_data["tokenString"]

        # Get the current date and time
        current_datetime = datetime.now()
        future_datetime = current_datetime + relativedelta(months=1)

        # Format it using strftime
        current_datetime = current_datetime.strftime("%Y-%m-%dT%H:%M:%S")
        future_datetime = future_datetime.strftime("%Y-%m-%dT%H:%M:%S")

        data = {
            "formValues": {
                "Check your bin day": {
                    "tokenString": {
                        "value": tokenString,
                    },
                    "UPRNForAPI": {
                        "value": user_uprn,
                    },
                    "formatDateToday": {
                        "value": current_datetime,
                    },
                    "formatDateTo": {
                        "value": future_datetime,
                    },
                }
            },
        }

        params = {
            "id": "5c8f869376376",
            "repeat_against": "",
            "noRetry": "false",
            "getOnlyTokens": "undefined",
            "log_id": "",
            "app_name": "AF-Renderer::Self",
            # unix_timestamp
            "_": str(int(time.time() * 1000)),
            "sid": sid,
        }
        r = s.post(API_URL, json=data, headers=headers, params=params)
        r.raise_for_status()

        data = r.json()

        rows_data = data["integration"]["transformed"]["rows_data"]
        if not isinstance(rows_data, dict):
            raise ValueError("Invalid data returned from API")

        # Extract each service's relevant details for the bin schedule
        for item in rows_data.values():
            if item["Name"]:
                Bin_Types = item["Name"].split("Empty Bin ")
                for Bin_Type in Bin_Types:
                    if Bin_Type:
                        dict_data = {
                            "type": Bin_Type.strip(),
                            "collectionDate": datetime.strptime(
                                item["Date"], "%Y-%m-%dT%H:%M:%S"
                            ).strftime(date_format),
                        }
                        bindata["bins"].append(dict_data)

        bindata["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )
        return bindata
